@@ -1,8 +1,12 @@
 
+from pyroute2.iproute import IPRoute
+from pyroute2.ipdb import IPDB
 from pyroute2.netlink.rtnl import IPRSocket
-from pyroute2.netlink.iproute import IPRoute
-from pyroute2.netlink.ipdb import IPDB
 from pyroute2.netlink.taskstats import TaskStats
+from pyroute2.netlink.ipq import IPQSocket
+from pyroute2.netlink.ipq import IPQ
+from pyroute2.netlink.generic import GenericNetlinkSocket
+from pyroute2.netlink.generic import GenericNetlink
 from pyroute2.iocore.iocore import IOCore
 from pyroute2.iocore import TimeoutError
 from pyroute2.netlink import NetlinkError
@@ -11,6 +15,10 @@ modules = [IPRSocket,
            IPRoute,
            IPDB,
            TaskStats,
+           IPQSocket,
+           IPQ,
+           GenericNetlinkSocket,
+           GenericNetlink,
            IOCore,
            TimeoutError,
            NetlinkError]
