@@ -1,11 +1,8 @@
-from pyroute2.netlink import Marshal
 from pyroute2.netlink import NLM_F_DUMP
 from pyroute2.netlink import NLM_F_REQUEST
-from pyroute2.netlink.generic import ctrlmsg
-from pyroute2.netlink.generic import NETLINK_GENERIC
-from pyroute2.netlink.generic import GENL_ID_CTRL
-from pyroute2.netlink.generic import CTRL_CMD_GETFAMILY
+from pyroute2.netlink import NETLINK_GENERIC
 from pyroute2.iocore.iocore import IOCore
+from pyroute2.netlink.nlsocket import Marshal
 
 
 class Netlink(IOCore):
@@ -71,23 +68,3 @@ class Netlink(IOCore):
                 del msg['header']
 
         return result
-
-
-class GenericNetlink(Netlink):
-
-    def __init__(self, proto, msg_class, pid=None):
-        Netlink.__init__(self, pid=pid)
-        self.marshal.msg_map[GENL_ID_CTRL] = ctrlmsg
-        self.prid = self.get_protocol_id(proto)
-        self.marshal.msg_map[self.prid] = msg_class
-
-    def get_protocol_id(self, prid):
-        msg = ctrlmsg()
-        msg['cmd'] = CTRL_CMD_GETFAMILY
-        msg['version'] = 1
-        msg['attrs'].append(['CTRL_ATTR_FAMILY_NAME', prid])
-        response = self.nlm_request(msg, GENL_ID_CTRL,
-                                    msg_flags=NLM_F_REQUEST)[0]
-        prid = [i[1] for i in response['attrs']
-                if i[0] == 'CTRL_ATTR_FAMILY_ID'][0]
-        return prid
