@@ -1,6 +1,6 @@
 
-from pyroute2.netlink.generic import nlmsg
-from pyroute2.netlink.generic import nla
+from pyroute2.netlink import nlmsg
+from pyroute2.netlink import nla
 
 # address attributes
 #
@@ -38,7 +38,8 @@ class ifaddrmsg(nlmsg):
                ('IFA_BROADCAST', 'ipaddr'),
                ('IFA_ANYCAST', 'ipaddr'),
                ('IFA_CACHEINFO', 'cacheinfo'),
-               ('IFA_MULTICAST', 'ipaddr'))
+               ('IFA_MULTICAST', 'ipaddr'),
+               ('IFA_FLAGS', 'uint32'))
 
     class cacheinfo(nla):
         fields = (('ifa_prefered', 'I'),
